/*
///////////////////////////////////////////////////////////////////////////////////

    Basic GPIO demo for the STM32 Blue Pill
    Blog Part 2

    Copyright (C) 2021 - Martin Kaselis (modelrailmusings.weebly.com)

///////////////////////////////////////////////////////////////////////////////////
*/

#include "stm32.h"

///////////////////////////////////////////////////////////////////////////////////

uint32_t milliSeconds = 0;      // Master uptime counter
uint32_t lastMilliSeconds = 0;

///////////////////////////////////////////////////////////////////////////////////

void initHardware()
{
    clockInit();        // 72MHz using 8MHz HSE & 9xPLL
    gpioInit();

    // Init I/Os
    gpioPinInit( GPIO_PC13, GPIO_MODE_OUTPUT ); // LED indicator

    systickInit();
}

///////////////////////////////////////////////////////////////////////////////////

// Called from interrupt vector in stm32_startup.c
void SysTick_Handler( void )
{
    // Master time counter, wraps after 49 days, so can ignore for this application
    ++milliSeconds;
}

// Delay until the next N millisecond tick.  This compensates for long pressing time in the main loop.
void timerDelay( uint32_t ms )
{
    while( milliSeconds - lastMilliSeconds < ms )
        ;
    lastMilliSeconds += ms;
}

///////////////////////////////////////////////////////////////////////////////////

int main( void )
{
    int i;

    initHardware();

    gpioPinWrite( GPIO_PC13, 1 ); // negative logic for the Blue Pill LED: 1=off

    i = 0;
    for(;;)
    {
        timerDelay( 20 );       // Delay till the next 20ms tick

        ++i;

        if( i == 40 )
        {
            gpioPinWrite( GPIO_PC13, 0 );
        }
        else if( i >= 50 )
        {
            gpioPinWrite( GPIO_PC13, 1 );
            i = 0;
        }
    }
}
