/*
///////////////////////////////////////////////////////////////////////////////////

    Partial STM32 library for GPIO demo for the STM32 Blue Pill
    Blog Part 2

    Copyright (C) 2021 - Martin Kaselis (modelrailmusings.weebly.com)

///////////////////////////////////////////////////////////////////////////////////
*/

#pragma once

#include <stdint.h>
#include <stdbool.h>

///////////////////////////////////////////////////////////////////////////////////////////

typedef struct
{
    uint32_t ACR;
} STM_FLASH;

extern STM_FLASH *FLASH;

#define FLASH_ACR_LATENCY_MASK  0x0003
#define FLASH_ACR_LATENCY_2     0x0002          // Two wait states, per datasheet

///////////////////////////////////////////////////////////////////////////////////////////

typedef struct
{
    uint32_t CR;
    uint32_t CFGR;
    uint32_t pad1[4];
    uint32_t APB2ENR;
} STM_RCC;

extern STM_RCC *RCC;

#define RCC_CR_HSEON            0x00010000
#define RCC_CR_HSERDY           0x00020000
#define RCC_CR_PLLON            0x01000000
#define RCC_CR_PLLRDY           0x02000000

#define RCC_CFGR_PPRE1_2        0x00000400
#define RCC_CFGR_PLLSRC         0x00010000
#define RCC_CFGR_PLLMULL9       0x001c0000
#define RCC_CFGR_SW_PLL         0x00000002
#define RCC_CFGR_SWS_PLL        0x00000008
#define RCC_CFGR_ADCPRE_DIV6    0x00008000 // Must be <= 14MHz, so 72MHz clock / 6 => 12 MHz

#define IOPC_EN                 0x00000010
#define IOPB_EN                 0x00000008
#define IOPA_EN                 0x00000004
#define ADC1EN                  0x00000200
#define AFIOEN                  0x00000001
#define USART1EN                0x00004000
#define SPI1EN                  0x00001000

///////////////////////////////////////////////////////////////////////////////////////////

typedef struct
{
    uint32_t CSR;
    uint32_t RVR;
    uint32_t CVR;
} STM_STK;

extern STM_STK *STK;

#define STK_CSR_COUNT           0x00010000
#define STK_CSR_PROCESSOR_CLK   0x00000004
#define STK_CSR_INTERRUPT       0x00000002
#define STK_CSR_ENABLE          0x00000001

///////////////////////////////////////////////////////////////////////////////////////////

typedef struct
{
    uint32_t CRL;
    uint32_t CRH;
    uint32_t IDR;
    uint32_t ODR;
    uint32_t BSRR;
} STM_GPIO;

extern STM_GPIO *GPIOA;
extern STM_GPIO *GPIOB;
extern STM_GPIO *GPIOC;

///////////////////////////////////////////////////////////////////////////////////////////

// GPIO bits: A0-15 = 0-15, B0-15 = 16-31, C0-15 = 32-47
typedef enum
{
    GPIO_PA0 = 0, GPIO_PA1, GPIO_PA2, GPIO_PA3, GPIO_PA4, GPIO_PA5, GPIO_PA6, GPIO_PA7,
    GPIO_PA8, GPIO_PA9, GPIO_PA10, GPIO_PA11, GPIO_PA12, GPIO_PA13, GPIO_PA14, GPIO_PA15,

    GPIO_PB0, GPIO_PB1, GPIO_PB2, GPIO_PB3, GPIO_PB4, GPIO_PB5, GPIO_PB6, GPIO_PB7,
    GPIO_PB8, GPIO_PB9, GPIO_PB10, GPIO_PB11, GPIO_PB12, GPIO_PB13, GPIO_PB14, GPIO_PB15,

    GPIO_PC0, GPIO_PC1, GPIO_PC2, GPIO_PC3, GPIO_PC4, GPIO_PC5, GPIO_PC6, GPIO_PC7,
    GPIO_PC8, GPIO_PC9, GPIO_PC10, GPIO_PC11, GPIO_PC12, GPIO_PC13, GPIO_PC14, GPIO_PC15
} GPIO_BIT;

typedef enum
{
    GPIO_MODE_INPUT = 0,        // input, floating
    GPIO_MODE_OUTPUT,           // output push-pull, max speed 10MHz
    GPIO_MODE_ANALOG,           // analog input, floating
    GPIO_MODE_AF_PUSHPULL       // alternate function output push-pull, max speed 10MHz
} GPIO_MODE;

///////////////////////////////////////////////////////////////////////////////////////////

extern void clockInit( void );
extern void gpioInit( void );
extern void systickInit( void );

extern void delayUsec( uint32_t usec );

extern void gpioPinInit( GPIO_BIT bit, GPIO_MODE mode );
extern void gpioPinWrite( GPIO_BIT bit, uint32_t on );
extern void gpioPinWriteMultiple( GPIO_BIT bit, uint32_t mask, uint32_t value );
extern uint32_t gpioPinRead( GPIO_BIT bit );
